% ---------------------------------------------------------------------
% Book:        SFS
% ---------------------------------------------------------------------
% Quantlet:    SFSarch
% ---------------------------------------------------------------------
% Description: SFSarch reads data for DAX and FTSE 100 from 1 Jan 
%              to 31 Dec 2007 and models the volatility processes for the 
%              DAX and FTSE 100 returns as an ARCH(6) process. 
%              Corresponds to exercise 13.2 in SFS
% ---------------------------------------------------------------------
% Usage:       -
% ---------------------------------------------------------------------
% Inputs:      none
% ---------------------------------------------------------------------
% Output:      Plots of the log-likelihood objective functions and of 
%              forecasts of the estimated volatility processes for the 
%              period from 1 January 2008 until 31  December 2008 using the
%              unconditional and the conditional volatility approach. 
%              Estimated parameters for the return and volatility 
%              equations.            
% ---------------------------------------------------------------------
% Example:     -
% ---------------------------------------------------------------------
% Author:      Andrija Mihoci  20090803
% ---------------------------------------------------------------------
%

clear 
clc
close all

load FTSE_DAX.dat;
DS = FTSE_DAX;
D = DS(:,1);                            % date
S = DS(:,2:43);                         % S(t)
s = log(S);                             % log(S(t))
r = s(2:end,:) - s(1:(end-1),:);        % r(t)
n = length(r);                          % sample size
t = 1:n;                                % time index, t
rdax = r(:,1);                          % DAX returns
rftse = r(:,22);                        % FTSE 100 returns


% ARCH(q) models for the volatility proces of DAX returns
for q = 1:15
  spec = garchset('R', 0, 'M', 0, 'VarianceModel', 'GARCH', 'P', 0, 'Q', q,'Display','off');
  [Coeff,Errors,LLF] = garchfit(spec,rdax);
  DAXcoef = Coeff;
  DAXL(q) = LLF;
end;
 
  
DAXL = DAXL';


% ARCH(q) models for the volatility proces of FTSE 100 returns
for q = 1:15
  spec = garchset('R', 3, 'M', 0, 'VarianceModel', 'GARCH', 'P', 0, 'Q', q,'Display','off');
  [Coeff,Errors,LLF] = garchfit(spec,rftse);
  FTSEcoef = Coeff;
  FTSEL(q) = LLF;
end;

FTSEL = FTSEL';


% Plots of the optimized log-likelihood function for different q
figure1 = figure('PaperType','a4letter','PaperSize',[20.98 29.68],'Name','DAX and FTSE 100, ARCH(q) models');

subplot1 = subplot(1,2,1,'Parent',figure1,'FontWeight','bold','FontSize',14);
box('on');
hold('all');

plot(DAXL,'Parent',subplot1,'LineWidth',2);

title('DAX','FontWeight','bold','FontSize',14);
xlabel('q','FontWeight','bold','FontSize',14);
ylabel('Log-likelihood','FontWeight','bold','FontSize',14);

subplot2 = subplot(1,2,2,'Parent',figure1,'FontWeight','bold','FontSize',14);
box('on');
hold('all');

plot(FTSEL,'Parent',subplot2,'LineWidth',2);
title('FTSE 100','FontWeight','bold','FontSize',14);
xlabel('q','FontWeight','bold','FontSize',14);
ylabel('Log-likelihood','FontWeight','bold','FontSize',14);


% ARCH(6) model for the volatility process of DAX returns
spec = garchset('R', 0, 'M', 0, 'VarianceModel', 'GARCH', 'P', 0, 'Q', 6,'Display','off');
[DAXCoeff,errors,LLF,innovations,sigmacon] = garchfit(spec,rdax);
h = 250;
sigmauncdax = sqrt(DAXCoeff.K/(1 - sum(DAXCoeff.ARCH(:))));
sigmacondax = sigmacon;
[sigmafdax,meanfdax] = garchpred(DAXCoeff,rdax,h); %Volatility forcast based on estimated parameters


% ARCH(6) model for the volatility process of FTSE 100 returns
spec = garchset('R', 3, 'M', 0, 'VarianceModel', 'GARCH', 'P', 0, 'Q', 6,'Display','off');
[FTSECoeff,Errors,LLF,innovations,sigmacon] = garchfit(spec,rftse);
h = 250;
sigmauncftse = sqrt(FTSECoeff.K/(1 - sum(FTSECoeff.ARCH(:))));
sigmaconftse = sigmacon;
[sigmafftse,meanfftse] = garchpred(FTSECoeff,rftse,h); %Volatility forcast based on estimated parameters

% Plots of the estimated and forecasted unconditional and conditional volatility processes
figure2 = figure('PaperType','a4letter','PaperSize',[20.98 29.68],'Name','DAX and FTSE 100 volatility processes');

axes1 = axes('Parent',figure2,'FontWeight','bold','FontSize',14);
subplot1 = subplot(1,2,1,'Parent',figure2,'FontWeight','bold','FontSize',14);
box('on');
hold('all');

plot([sigmacondax;sigmafdax],'Parent',subplot1,'LineWidth',2);
title('DAX','FontWeight','bold','FontSize',13);
xlabel('Time','FontWeight','bold','FontSize',13);
ylabel('Volatility','FontWeight','bold','FontSize',13);
hold('on');

plot([0 size([sigmacondax;sigmafdax],1)],[sigmauncdax sigmauncdax],'red');
hold('on');
plot([2607,2607],[0,0.06],'k');

axes2 = axes('Parent',figure2,'FontWeight','bold','FontSize',14);
subplot2 = subplot(1,2,2,'Parent',figure2,'FontWeight','bold','FontSize',14);
box('on');
hold('all');

plot([sigmaconftse;sigmafftse],'Parent',subplot2,'LineWidth',2);
title('FTSE 100','FontWeight','bold','FontSize',14);
xlabel('Time','FontWeight','bold','FontSize',14);
ylabel('Volatility','FontWeight','bold','FontSize',14);
hold('on');

plot([0 size([sigmaconftse;sigmafftse],1)],[sigmauncftse sigmauncftse],'red');
hold('on');
plot([2607,2607],[0,0.06],'k');

%Display estimated parameter for DAX return and volatility process
disp('Estimated parameters for the return process - DAX')
disp('    epsilon')
disp([DAXCoeff.C])

disp('Estimated parameters for the volatility equation - DAX')
disp('      t-1      t-2        t-3       t-4       t-5       t-6')
disp([DAXCoeff.ARCH])
disp('      omega')
disp([DAXCoeff.K])

%Display estimated parameter for FTSE 100 return and volatility process
disp('Estimated parameters for the return process - FTSE 100')
disp('      t-1      t-2        t-3')
disp([FTSECoeff.AR])
disp('    epsilon')
disp([FTSECoeff.C])

disp('Estimated parameters for the volatility equation - FTSE 100')
disp('      t-1      t-2        t-3       t-4       t-5       t-6')
disp([FTSECoeff.ARCH])
disp('      omega')
disp([FTSECoeff.K])
