% ---------------------------------------------------------------------
% Book:        SFS
% ---------------------------------------------------------------------
% Quantlet:    SFSriskreversal
% ---------------------------------------------------------------------
% Description: SFSriskreversal compares the prices of the risk reversal
%              functions for the following implied volatility
%              curves, given as a function of strike price:
%              (i)  f1(K) = 0.000167 x K^2 - 0.03645 x K + 2.080
%              (ii) f2(K) = 0.000167 x K^2 - 0.03645 x K + 2.090
%              (iii)f3(K) = 0.000167 x K^2 - 0.03517 x K + 1.952.
%              The plots compare the functions f2 and f3 to the function
%              f1.
%              Corresponds to exercise 17.4 in SFS.
% ---------------------------------------------------------------------
% Usage:       -
% ---------------------------------------------------------------------
% Inputs:      S0     - Closing price of underlying at time t_0
%              strike - range of strike prices
%              tau    - time to maturity
%              r      - interest rate
% ---------------------------------------------------------------------
% Output:      The prices of risk reversal for functions f1, f2 and f3.
%              Comparison of f1 and f2 and of f1 and f3.
% ---------------------------------------------------------------------
% Example:     -
% ---------------------------------------------------------------------
% Author:      Szymon Borak 20090731
% ---------------------------------------------------------------------

close all
clear
clc

%inputs
S0 = 100;
strike = 85:1:115;
tau = 0.25;
r=0.02;

%define risk reversal
i1=1;i2=31;

%define functions
f1= @(x)0.000167*x.*x-0.03645 *x + 2.08;   %initial
f2= @(x)0.000167*x.*x-0.03645 *x + 2.09;   %shifted
f3=@(x)(0.000167*x.*x-0.03517 *x + 1.952); %tilt


%calculate
ivsmileK = f1(strike);
[call1 put1] = blsprice(S0,strike(i1),r,tau,ivsmileK(i1));
[call2 put2] = blsprice(S0,strike(i2),r,tau,ivsmileK(i2));
RR1 = put1-call2;

ivsmileK = f2(strike);
[callS1 putS1] = blsprice(S0,strike(i1),r,tau,ivsmileK(i1));
[callS2 putS2] = blsprice(S0,strike(i2),r,tau,ivsmileK(i2));
RR2 = putS1-callS2;

ivsmileK = f3(strike);
[callS1 putS1] = blsprice(S0,strike(i1),r,tau,ivsmileK(i1));
[callS2 putS2] = blsprice(S0,strike(i2),r,tau,ivsmileK(i2));

RR3 = putS1-callS2;

%final results
disp('Prices of risk reversal')
disp('      f1        f2        f3')
disp([RR1,RR2,RR3])

%difference f1 vs. f2
figure
plot(strike,f1(strike),'linewidth',2.5, 'color',[0,0,1])
hold on
plot(strike,f2(strike),'linewidth',2.5,'color',[0,0,1],'linestyle','--')
hold off
title('f_1 vs. f_2')
xlabel('Strike')
ylabel('\sigma_{imp}')
ylim([0.08,0.2])

%difference f1 vs. f3
figure
plot(strike,f1(strike),'linewidth',2.5, 'color',[0,0,1])
hold on
plot(strike,f3(strike),'linewidth',2.5,'color',[0,0,1],'linestyle','--')
hold off
title('f_1 vs. f_3')
xlabel('Strike')
ylabel('\sigma_{imp}')
ylim([0.08,0.2])