% ---------------------------------------------------------------------
% Book:        SFS
% ---------------------------------------------------------------------
% Quantlet:    SFSstickycall
% ---------------------------------------------------------------------
% Description: SFSstickycall plots call option prices as a function of strikes
%              for 85<K<115. The implied volatility function may be
%              fixed to the strike prices (sticky strike) or moneyness K/S
%              (sticky moneyness). Compares the relative difference of both
%              approaches.
%              Corresponds to exercise 17.3 in SFS
% ---------------------------------------------------------------------
% Usage:      -
% ---------------------------------------------------------------------
% Inputs:     S0 - Underlying price at time t_0
%             S1 - Underlying price at time t_1
%             strike - range of strike prices
%             tau - time to maturity
%             r - interest rate
% ---------------------------------------------------------------------
% Output:     Plots call option prices with two different stickiness
%             assumptions.
% ---------------------------------------------------------------------
% Example:    An example is produced for S0 = 100, strike = 85:1:115,
%             tau = 0.25 and r=0.02.
% ---------------------------------------------------------------------
% Author:     Szymon Borak 20090731
% ---------------------------------------------------------------------

clear
clc
close all

%inputs
S0 = 100;
S1 = 105;
strike = 85:1:115;
tau = 0.25;
r=0.02;

f= @(x)0.000167*x.*x-0.03645 *x + 2.08;

%calculate
ivsmileK = f(strike);
callinit = blsprice(S0,strike,r,tau,ivsmileK);

callstickystrike = blsprice(S1,strike,r,tau,ivsmileK);

ivsmilemon = f(strike*S0/S1);
callstickymon = blsprice(S1,strike,r,tau,ivsmilemon);

figure
plot(strike,callinit,strike,callstickystrike,strike,callstickymon)
title('Call prices as a function of strikes')
xlabel('Strike')
ylabel('Call prices')

figure
plot(strike,(callstickystrike-callstickymon)./callstickystrike)
title('Relative difference between call prices for different stickyness assumptions')
xlabel('Strike')

figure
plot(strike,ivsmileK,strike,ivsmilemon)
title('Implied volatility for different stickiness assumptions')
xlabel('Strike')
ylabel('\sigma_{imp}')