# ---------------------------------------------------------------------
#  Book:        SFS
# ---------------------------------------------------------------------
#  Quantlet:    IBTcrr
# ---------------------------------------------------------------------
#  Description: computes European option prices using a binomial tree for
#      	        assets with no/continuous divid}s assuming a constant 
#               implied volatility.
# ---------------------------------------------------------------------
# Usage:        C = IBTcrr(S0, K, r, sig, T, n, flag)
#---------------------------------------------------------------------
# Inputs:       Price of underlying asset, Strike price, Interest rate, 
#               Implied Volatility, Time to expiration, Number of steps
#               in the tree, Flag for Call/Put option
# ---------------------------------------------------------------------
# Output:       Price of an European option.
# --------------------------------------------------------------------- 
# Example:      C = IBTcrr(100, 80, 0.03, 0.1, 1, 4, 1)
#               option flag (1 for call, 0 for put)
#---------------------------------------------------------------------
# Result:       C =
#                      22.3543
#---------------------------------------------------------------------
# Authors:      Wolfgang Haerdle, Ying Chen, Alena Mysickova 20080628
# --------------------------------------------------------------------
IBTcrr<-function(s0, k, i, sig, t, n, flag)  {
	type=1                  # 0 is American/1 is European
	div=0 
	if (s0<=0) {
  		print('IBTcrr: Price of Underlying Asset should be positive! Please print again')
   		print('s0=')
		s0=scan()
	}
  	if (k<0) {
  		print('IBTcrr: Exercise price couldnot be negative! Please print again')
     		print('k=')
		k=scan()	
  	}
  	if (sig<0) {
  		print('IBTcrr: Volatility should be positive! Please print again')
     		print('sig=')
		sig=scan()
  	}
  	if (t<=0) {
  		print('IBTcrr: Time to expiration should be positive! Please print again')
    		print('t=') 
		t=scan()
  	}
  	if (n<1) {
  		print('IBTcrr: Number of steps should be at least equal to 1! Please print again')
  		n=print('n=')	
		n=scan()
  	}
  	if (n>150)	{   # Constraint of n, otherwise it will take too much time
  		print('IBTcrr: Could you please choose a smaller n? Please print again')
  		print('n=')
		n=scan()
  	}
	dt=t/n 				                  # Interval of step
	u=exp(sig*sqrt(dt))                   # Up movement parameter u      
	d=1/u                                 # Down movement parameter d      
	b=i-div                               # Costs of carry 
	p=0.5+0.5*(b-sig^2/2)*sqrt(dt)/sig    # Probability of up movement
	s=matrix(1,n+1,n+1)*s0
	un=rep(1,n+1)-1 
	un[n+1]=1
	dm=t(un)
	um=matrix(0,0,n+1)
	j=1

	while (j<n+1) {
		d1=cbind(t(rep(1,n-j)-1), t((rep(1,j+1)*d)^(seq(1,j+1)-1)))
		dm=rbind(dm,d1)                                    # Down movement dynamics
    		u1=cbind(t(rep(1,n-j)-1),t((rep(1,j+1)*u)^(seq(j,0))))
		um=rbind(um,u1)    		                        # Up movement dynamics
		j=j+1;
	}
	
	um=t(rbind(un,um))
	dm=t(dm)
	s=s[1,1]*um*dm;                                     # Stock price development 	 
	s=s[nrow(s):1,] 	                                       # Rearangement
	opt=matrix(0,nrow(s),ncol(s))
  
 	if ((flag == 1) && (type==0)) {                                # Option is a american call 
  		opt[,n+1] = pmax(s[,n+1]-k,0)                      # Determine option values from prices 
  		for (j in seq(n,1)) { 
    			l = seq(1,j,1) 
			# Probable option values discounted back one time step 
    			discopt = ((1-p)*opt[l,j+1]+p*opt[l+1,j+1])*exp(-b*dt) 
			# Option value is max of current price - X or discopt 
    			opt[,j] = rbind(t(t(pmax(s[1:j,j]-k,discopt))),t(t(rep(0,n+1-j,1))))
  		} 
		
    		American_Call_Price = opt[nrow(opt):1,]
		price=American_Call_Price(n+1,1)
	}
 	if ((flag == 1) && (type==1)) {                            # Option is a european call 
  		opt[,n+1] = pmax(s[,n+1]-k,0)                      # Determine option values from prices 
  		for (j in seq(n,1)) { 
    			l = seq(1,j,1) 
			# Probable option values discounted back one time step 
    			discopt = ((1-p)*opt[l,j+1]+p*opt[l+1,j+1])*exp(-b*dt); 
			# Option value
    			opt[,j] = rbind(t(t(discopt)),t(t(rep(0,n+1-j))))
  		} 
    		European_Call_Price = opt[nrow(opt):1,]
    		price=European_Call_Price[n+1,1]
	}
	if ((flag == 0) && (type==0)) {                             # Option is an american put 
  		opt[,n+1] = pmax(k-s[,n+1],0)                      # Determine option values from prices 
  		for (j in seq(n,1)) { 
    			l = seq(1,j,1) 
			# Probable option values discounted back one time step 
    			discopt = ((1-p)*opt[l,j+1]+p*opt[l+1,j+1])*exp(-b*dt); 
			# Option value is max of X - current price or discopt
			opt[,j] = rbind(t(t(pmax(k-s[l,j],dicopt))),t(t(rep(0,n+1-j)))) 
    			
  		}
    		American_Put_Price = opt[nrow(opt):1,]
		price=American_Put_Price(n+1,1)
	 }
  	if ((flag == 0) && (type == 1)) {                           # Option is a european put 
  		opt[,n+1] = pmax(k-s[,n+1],0)                      # Determine option values from prices 
  		for (j in seq(n,1)) { 
    			l = seq(1,j,1) 
			# Probable option values discounted back one time step 
    			discopt = ((1-p)*opt[l,j+1]+p*opt[l+1,j+1])*exp(-b*dt) 
			# Option value 
    			opt[,j] = rbind(t(t(discopt)),t(t(rep(0,n+1-j))))
    		} 
    		European_Put_Price = opt[nrow(opt):1,]
    		price=European_Put_Price[n+1,1]
	}
	price
 }
