# ---------------------------------------------------------------------
# Book:         SFS
# ---------------------------------------------------------------------
# Quantlet:     SFShullhedgeratio
# ---------------------------------------------------------------------
# Description:  Stopp loss and cost of hedging. Refers to exercise 6.2 in 
# SFS.
# ---------------------------------------------------------------------
# Usage:        -
# ---------------------------------------------------------------------
# Inputs:       St - stock price
#               Kp - strike price for long put
#               Kc - strike price for short call
# ---------------------------------------------------------------------
# Output:       BS price of the call and the cost of hedging 
# ---------------------------------------------------------------------
# Example:      an example is produced for the values:
#               K=50, R share=100000, volatility=0.2, r=5%
# ---------------------------------------------------------------------
# Author:        
# Date 20100202
# ---------------------------------------------------------------------
#
# requires package fOption
# fOption nests packages timeDate, timeSeries, fBasics and MASS
# 
#  

rm(list=ls())
 


# case: call

 
S0 = 49           # Starting price
T = 20/52          # Time to maturity
si = 0.2        # Volatility
mu = 0.13            # Drift
r = 0.05           # Interest rate
K = 50             # Strikeprice
NSteps = 2000 # Number of steps
NRepl = 500       # Number of replications

NumObs = c(4,5,10,20,50,100)

#Calculate the BS Price of the call
# Use package fOptions
install.packages("fOptions")
library('fOptions')
CallOpt=GBSOption(TypeFlag="c",S=S0,X=K,Time=T,r=r,b=0,sigma=si)
Call=attr(CallOpt,"price")


###############################
# Optional: Call blackscholes.r
# source("blackscholes.r") 
#CallOpt=blackscholes(S0,K,T,r,si)
#Call=CallOpt[1]
###############################

## Simulate the paths for the underlying stock
St = matrix(0,NRepl,NSteps+1)
St[,1] = rep(S0,NRepl)  #All paths start with S0
dt = T/NSteps               #Discretize the time to maturity into discrete time steps corresponding to NSteps in T
drift = (mu-0.5*si^2)*dt #Calculate the drift rate
diffusion= si*sqrt(dt)   #Calculate the diffusion process rate


for (i in seq(1,NRepl,1)) {
    for (j in seq(1,NSteps,1)) {
        St[i,j+1]=St[i,j]*exp(drift + diffusion*rnorm(1,0,1))
    }
}

## Calculate the hedging cost at the specified observation times

#Determine size of time steps for the observations
ObsDensity = NSteps/NumObs

#Determine the discount factors for all t =0,1,2,...,1000
DiscountFactors = exp(-r*seq(0,NSteps,1)*dt)
# DiscountFactors=zeros(1,1001)+1;

Cost = matrix(0,NRepl,length(NumObs))

L=matrix(0,length(ObsDensity),length(ObsDensity))

for (m in seq(1,length(ObsDensity),1)) {
    for (k in seq(1,NRepl,1)) {

        CashFlows = matrix(0,NSteps+1,1)

        if (St[k,1]>=K) {
            Covered = 1
            CashFlows[1] = -St[k,1]
	  }
        else {
            Covered = 0
        }

        for (t in seq(1,(NSteps+1),ObsDensity[m])) {
            if ((Covered == 1) && (St[k,t] < K)) {
                # Sell
                Covered = 0
                CashFlows[t] = St[k,t]
		}
            if ((Covered == 0) && (St[k,t] > K)) {
                # Buy
                Covered = 1
                CashFlows[t] = -St[k,t]
            }
        }

        if (St[k,NSteps + 1] >= K) {
            # Option is exercised
            CashFlows[NSteps + 1] = CashFlows[NSteps + 1] + K
        }

        Cost[k,m] = -DiscountFactors%*%CashFlows
        
    }
    V=apply(Cost,2,var)
    L[m,]=sqrt(V)/Call
}

print('Performance measure L:')
X=colMeans(L)


print(X)